local function Engine_OnKeyDown(self, caller, key)
	if not self.running or not self.protagonist then return end
	if 		key == "LEFT" then
		self.protagonist.direction = "left"
		self.keys.leftDown = true;
		self.keys.rightDown = false;
	elseif	key == "RIGHT" then
		self.protagonist.direction = "right"
		self.keys.leftDown = false;
		self.keys.rightDown = true;
	elseif key == "SPACE" then
		self.keys.spaceFired = true;
	end
end

local function Engine_OnKeyUp(self, caller, key)
	if not self.running or not self.protagonist then return end
	if 		key == "LEFT" then
		self.keys.leftDown = false;
	elseif	key == "RIGHT" then
		self.keys.rightDown = false;
	elseif key == "SPACE" then
		self.keys.spaceFired = false;
	end
end

local function Engine_SetSpeed(self, speed)
	if not self.running then return end
	self.gameSpeed = tonumber(speed) or self.gameSpeed;
end

local function Engine_Update(self, ...)
	local speed = 0;
	if self.keys.leftDown == true then speed = -(self.protagonist.speed); end
	if self.keys.rightDown == true then speed = (self.protagonist.speed); end
	self.protagonist.forceH = speed;
	if self.keys.spaceFired and self.protagonist:OnGround() then
		self.protagonist:Jump()
	end

	local now = GetTime();
	if (now - self.lastUpdate) >= (1/self.gameSpeed) then
		self.world:Step(1/self.gameSpeed);
		self.lastUpdate = now;
	end
end



local function Engine_LoadWorld(self, ...)
	if not self.world then
		self.world = Gnomario_World_Create(self.frame,480,320);
	end
	self.world:Clear();
end

local function Engine_Start(self)
	self.running = true;
	self.lastUpdate = GetTime();
end

local function Engine_SetProtagonist(self, mob)
	self.protagonist = mob;
end

function Gnomario_Engine(parent)
	local engine = {};
	engine.sprites = {};
	engine.keys = {};
	engine.frame = CreateFrame("Frame");
	engine.frame:SetParent(parent);
	engine.frame:EnableKeyboard(true);
	engine.frame:SetAllPoints(parent);
	engine.frame:SetScript("OnKeyDown", function(...) Engine_OnKeyDown(engine,...) end);
	engine.frame:SetScript("OnKeyUp", function(...) Engine_OnKeyUp(engine,...) end);
	engine.frame:SetScript("OnUpdate", function(...) Engine_Update(engine, ...) end);
	engine.gameSpeed = 25;
	engine.running = false;
	engine.SetSpeed = Engine_SetSpeed;
	engine.LoadWorld = Engine_LoadWorld;
	engine.Start = Engine_Start;
	engine.SetProtagonist = Engine_SetProtagonist;
	return engine;
end
